#!/usr/bin/env python
# encoding: utf-8

###########################################################################
# Copyright 2013. All rights reserved.                                    #
# Institute of Measurement and Control Systems                            #
# Karlsruhe Institute of Technology, Germany                              #
#                                                                         #
# This file is part of triTrack2.                                         #
# Authors:  Philip Lenz                                                   #
#           Please send any bugreports to lenz@kit.edu                    # 
###########################################################################

import sys, os, shutil, argparse
from PIL import Image, ImageDraw, ImageFont, ImageChops
from subprocess import call

def createVideo(input_dir, video_res):
    """Merges images for triTrack2 output directory structure and creates video."""

    print "Init",
    # params
    img_ext = "jpg"
    bitrate = 2800
    fps = 15
    print "- done"

    # list directories in input dir to determine layout
    print "Detect figures",
    subdirs = [os.path.split(x[0])[-1] for x in os.walk(input_dir)]
    figures = {}
    dim = {}
    if "img" in subdirs:
        figures["img"] = True
        img_file = os.path.join(input_dir,"img","%06d.jpg" % 0)
        dim["img"] = Image.open(img_file).size
    if "ics" in subdirs:
        figures["ics"] = True
        img_file = os.path.join(input_dir,"ics","%06d.jpg" % 0)
        dim["ics"] = Image.open(img_file).size
    if "wcs" in subdirs:
        figures["wcs"] = True
        img_file = os.path.join(input_dir,"wcs","%06d.jpg" % 0)
        dim["wcs"] = Image.open(img_file).size

    if set(("img","ics","wcs")).issubset((set(figures.keys()))):
        blank_img_dim = [max(dim["img"][0], dim["ics"][0], dim["wcs"][0]),
                         dim["img"][1] + max(dim["ics"][1], dim["wcs"][1])] 
    else:
        raise NameError("Layout not implemented yet.")
    print "- done"

    print "Start loop"
    img_idx = -1
    font = ImageFont.truetype("/usr/share/fonts/truetype/ttf-dejavu/DejaVuSans-Bold.ttf",32)
    tmp_dir = os.path.join(input_dir, "tmp")
    if not os.path.exists(tmp_dir):
        os.makedirs(tmp_dir)
    while True:
        img_idx+=1
        img = {}
        try:
            for k in figures.keys():
                img_file = os.path.join(input_dir,k,"%06d.jpg" % img_idx)
                img[k] = Image.open(img_file)
            print "  . process image %d" % img_idx
        except IOError:
            break
        # arrange images according to existing data
        if set(("img","ics","wcs")).issubset((set(figures.keys()))):
            # create image and place triTrack2 output
            blank_image = Image.new("RGB", blank_img_dim)
            blank_image.paste(img["img"], (0,0))
            offset = dim["img"][0]-(dim["ics"][0]+dim["wcs"][0])
            blank_image.paste(img["ics"], (0+offset/2,dim["img"][1]))
            blank_image.paste(img["wcs"], (dim["ics"][0]+offset/2,dim["img"][1]))

            # add information to image
            background = Image.new("RGBA", blank_img_dim)
            draw = ImageDraw.Draw(background)
            # frame number, ICS, WCS
            drawText(blank_image,draw,background,(0,0),"Frame %d" % img_idx, font)
            drawText(blank_image,draw,background,(0,dim["img"][1]),"image coordinate system",font,dim["ics"])
            drawText(blank_image,draw,background,(dim["ics"][0],dim["img"][1]),"world coordinate system",font,dim["wcs"])

            # resize image to video resolution and save/show
            blank_image.thumbnail(video_res)
            outfile = os.path.join(tmp_dir, "%06d.%s" % (img_idx, img_ext))
            blank_image.save(outfile)
        else:
            raise NameError("Layout not implemented yet.")

    # create avi
    optimal_bitrate = 50 * 25 * blank_image.size[0] * blank_image.size[1] / 256
    call(["mencoder", 
          ("mf://" +  tmp_dir+"/"),
           "-mf", ("fps=" + str(fps) + ":type=%s" % img_ext),
           "-ovc", "xvid", "-xvidencopts", "bitrate=%d" % bitrate,
           "-nosound",
           "-o", (os.path.join(input_dir, "tritrack.avi"))]
        )

    print "Finish"
    # clean up
    #shutil.rmtree(tmp_dir)
    print "All Done. Exiting..."

def drawText(image,draw,background,pos,textstr,font, dim=None):
    textsize = draw.textsize(textstr,font=font)
    if dim is not None:
        offset_x = (dim[0]-textsize[0])/2
        pos = ( pos[0]+offset_x, pos[1] )
    draw.rectangle([pos,(pos[0]+textsize[0],pos[1]+textsize[1])], fill=(20,20,20,200))
    draw.text(pos, textstr, font=font, fill=(0,255,0,255))
    image.paste(background,mask=background)

if __name__=="__main__":
    print "dbg"
    parser = argparse.ArgumentParser(description="Create videos for triTrack2.")
    parser.add_argument("-d","--dir", dest="input_dir", required=True,
                        help="specifies the base directory of triTrack2 output")
    parser.add_argument("--video-res", dest="video_res", required=False, default=(1024,768),
                        help="resolution of the output video, preserves aspect ratio.")
    args = vars(parser.parse_args()) 
    createVideo(input_dir=args["input_dir"], video_res=args["video_res"])
